import _mapValues from 'lodash/mapValues';
import _omit from 'lodash/omit';
import _sumBy from 'lodash/sumBy';
import _toString from 'lodash/toString';
import _map from 'lodash/map';
import { format } from 'date-fns';
import fetch from 'isomorphic-fetch';

function _typeof(obj) {
  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

// copy from https://github.com/vkiryukhin/jsonfn
var toJSON = function toJSON(value) {
  if (value instanceof Function || typeof value === 'function') {
    var fnBody = value.toString();

    if (fnBody.length < 8 || fnBody.substring(0, 8) !== 'function') {
      // this is ES6 Arrow Function
      return "_EsFunc_".concat(fnBody);
    }

    return fnBody;
  }

  return value;
};

var fromJSON = function fromJSON(value) {
  if (typeof value === 'string' && value.length >= 8) {
    var prefix = value.substring(0, 8);

    if (prefix === 'function') {
      // eslint-disable-next-line no-eval
      return eval("(".concat(value, ")"));
    }

    if (prefix === '_EsFunc_') {
      // eslint-disable-next-line no-eval
      return eval(value.slice(8));
    }
  }

  return value;
};

var parseObject = function parseObject(value) {
  if (value) {
    var type = _typeof(value);

    if (type === 'string') {
      return fromJSON(value);
    }

    if (type === 'object' && !Array.isArray(value)) {
      return _mapValues(value, parseObject);
    }
  }

  return value;
};

var jsonFn = {
  toJSON: toJSON,
  fromJSON: fromJSON,
  parseObject: parseObject,
  stringify: function stringify(obj) {
    return JSON.stringify(obj, function (key, value) {
      return toJSON(value);
    });
  },
  parse: function parse(str) {
    return JSON.parse(str, function (key, value) {
      return fromJSON(value);
    });
  }
};

var util = {
  map: _map,
  toString: _toString,
  sumBy: _sumBy,
  dateFnsFormat: format // const specialKeys = ['render', 'layout', 'baseKey', '_base']

};
var render = function render(data, template) {
  var templateRender = template.render;

  if (templateRender) {
    if (typeof templateRender === 'function') {
      return templateRender.call(util, data, _omit(template, 'render'), util);
    }

    return templateRender;
  } // render list of sub-template which ordered by layout


  if (template.layout) {
    return _map(template.layout, function (k) {
      return render(data, _objectSpread({}, template, template[k]));
    });
  }

  return template;
};
util.render = render;

var _extendDeep = function _extendDeep(a, b) {
  if (b && _typeof(a) === 'object' && !Array.isArray(a)) {
    return _objectSpread({}, b, {
      _base: b
    }, _mapValues(a, function (subA, key) {
      var subB = b[subA.baseKey || key];
      return subB ? _extendDeep(subA, subB) : subA;
    }));
  }

  return a;
};

var extend = function extend(a, b) {
  var template = _extendDeep(a, b); // handle main block


  var main = template.main || template.Main;

  if (main) {
    return _objectSpread({}, _omit(template, 'main', 'Main'), main);
  }

  return template;
};
util.extend = extend;

var HOST = 'https://ibill.today';

var _renderPdf = function _renderPdf(prop) {
  return fetch("".concat(prop.HOST, "/pdf-render/MY-DOC.pdf"), {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      data: prop.data,
      templateId: prop.templateId
    })
  }).then(function (res) {
    return res.blob();
  });
};

var defaultProp = {
  HOST: HOST // prop = { templateId, data, HOST }

};
var renderPdf = function renderPdf(_prop) {
  return _renderPdf(_objectSpread({}, defaultProp, _prop));
};

export { render, extend, jsonFn as Json, renderPdf };
